package com.winhc.widget.util;

import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.database.Cursor;
import android.net.Uri;
import android.os.Build;
import android.provider.MediaStore;
import android.support.v4.content.FileProvider;

import java.io.File;
import java.util.List;

/**
 * 针对Android 7.0及以上机型打开Uri适配
 * <pre>
 *     name  文件别名
 *     path 相对于当前标签的根目录 用.即可
 * </pre>
 * 根目录  /
 * <pre>
 *     <root-path
 *      name="root"
 *      path="." />
 * </pre>
 * 外置路径 /storage/emulate/0
 * <pre>
 *  <external-path
 *  name="external"
 *  path="." />
 * </pre>
 * 外置媒体缓存路径 /storage/emulated/0/Android/media/<包名>
 * <pre>
 *     <external-media-path
 *     name="external-media-path"
 *     path="." />
 * </pre>
 * App 文件路径 /data/data/<包名>/files
 * <pre>
 *     <files-path
 *     name="files"
 *     path="." />
 * </pre>
 * App 缓存路径 /data/data/<包名>/cache
 * <pre>
 *     <cache-path
 *     name="cache"
 *     path="." />
 * </pre>
 * App 外置文件路径 /storage/emulate/0/Android/data/<包名>/files
 * <pre>
 *     <external-files-path
 *     name="external_file_path"
 *     path="." />
 * </pre>
 * App 外置缓存路径 /storage/emulate/0/Android/data/<包名>/cache
 * <pre>
 *     <external-cache-path
 *     name="external_cache_path"
 *     path="."
 * </pre>
 */
public class FileProviderHelper {
    public static String getAuthority(Context context) {
        return context.getPackageName() + ".fileProvider";
    }

    public static Uri getUriForFile(Context context, File file) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
            //大于Android 10.0
            ContentValues contentValues = new ContentValues(1);
            contentValues.put("_data", file.getAbsolutePath());
            return context.getContentResolver().insert(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, contentValues);
        } else if (Build.VERSION.SDK_INT < Build.VERSION_CODES.N) {
            //小于Android 7.0
            return Uri.fromFile(file);
        }
        return FileProvider.getUriForFile(context, getAuthority(context), file);
    }

    /**
     * 根据图片查询URI地址
     *
     * @param context
     * @param imageFile
     * @return
     */
    public static Uri queryMediaImageUri(Context context, File imageFile) {
        String filePath = imageFile.getAbsolutePath();
        Cursor cursor = context.getContentResolver().query(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, new String[]{"_id"}, "_data=? ", new String[]{filePath}, (String) null);
        if (cursor != null && cursor.moveToFirst()) {
            int id = cursor.getInt(cursor.getColumnIndex("_id"));
            Uri baseUri = Uri.parse("content://media/external/images/media");
            return Uri.withAppendedPath(baseUri, "" + id);
        } else if (imageFile.exists()) {
            ContentValues values = new ContentValues();
            values.put("_data", filePath);
            return context.getContentResolver().insert(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, values);
        } else {
            return null;
        }
    }

    /**
     * 是否有权限
     *
     * @param context
     * @param intent
     * @param uri
     * @param writeAble
     */
    public static void grantPermissions(Context context, Intent intent,
                                        Uri uri, boolean writeAble) {
        int readUriPermission = Intent.FLAG_GRANT_READ_URI_PERMISSION;
        if (writeAble) {
            readUriPermission = readUriPermission | Intent.FLAG_GRANT_WRITE_URI_PERMISSION;
        }
        intent.addFlags(readUriPermission);
        List<ResolveInfo> resolveInfoList = context.getPackageManager().queryIntentActivities(intent, PackageManager.MATCH_DEFAULT_ONLY);

        for (ResolveInfo resolveInfo : resolveInfoList) {
            String packageName = resolveInfo.activityInfo.packageName;
            context.grantUriPermission(packageName, uri, readUriPermission);
        }
    }
}
