package com.winhc.widget.util;

import android.content.Context;
import android.support.annotation.Nullable;
import android.support.v4.view.ViewCompat;
import android.support.v7.widget.GridLayoutManager;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.LinearSmoothScroller;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.SimpleItemAnimator;
import android.support.v7.widget.StaggeredGridLayoutManager;
import android.view.View;
import android.widget.AbsListView;
import android.widget.ScrollView;

/**
 * Create LayoutManager by Lee at 2021/6/30 11:15 AM
 **/
public final class RecyclerViewFactory {
    /**
     * 判断目标View是否滑动到顶部还能否继续滑动
     *
     * @return
     */
    public boolean isChildScrollToTop(View mTarget) {
        if (android.os.Build.VERSION.SDK_INT < 14) {
            if (mTarget instanceof AbsListView) {
                final AbsListView absListView = (AbsListView) mTarget;
                return !(absListView.getChildCount() > 0 && (absListView
                        .getFirstVisiblePosition() > 0 || absListView
                        .getChildAt(0).getTop() < absListView.getPaddingTop()));
            } else {
                return !(mTarget.getScrollY() > 0);
            }
        } else {
            return !ViewCompat.canScrollVertically(mTarget, -1);
        }
    }

    /**
     * 是否滑动到底部
     *
     * @return
     */
    public boolean isScrollToBottom(View mTarget) {
        if (isChildScrollToTop(mTarget)) {
            return false;
        }
        if (mTarget instanceof RecyclerView) {
            RecyclerView recyclerView = (RecyclerView) mTarget;
            RecyclerView.LayoutManager layoutManager = recyclerView.getLayoutManager();
            int count = recyclerView.getAdapter().getItemCount();
            if (layoutManager instanceof LinearLayoutManager && count > 0) {
                LinearLayoutManager linearLayoutManager = (LinearLayoutManager) layoutManager;
                return linearLayoutManager.findLastCompletelyVisibleItemPosition() == count - 1;
            } else if (layoutManager instanceof StaggeredGridLayoutManager) {
                StaggeredGridLayoutManager staggeredGridLayoutManager = (StaggeredGridLayoutManager) layoutManager;
                int[] lastItems = new int[2];
                staggeredGridLayoutManager
                        .findLastCompletelyVisibleItemPositions(lastItems);
                int lastItem = Math.max(lastItems[0], lastItems[1]);
                return lastItem == count - 1;
            }
            return false;
        } else if (mTarget instanceof AbsListView) {
            final AbsListView absListView = (AbsListView) mTarget;
            int count = absListView.getAdapter().getCount();
            int firstPos = absListView.getFirstVisiblePosition();
            if (firstPos == 0 && absListView.getChildAt(0).getTop() >= absListView
                    .getPaddingTop()) {
                return false;
            }
            int lastPos = absListView.getLastVisiblePosition();
            return lastPos > 0 && count > 0 && lastPos == count - 1;
        } else if (mTarget instanceof ScrollView) {
            ScrollView scrollView = (ScrollView) mTarget;
            View view = scrollView
                    .getChildAt(scrollView.getChildCount() - 1);
            if (view != null) {
                int diff = (view.getBottom() - (scrollView.getHeight() + scrollView
                        .getScrollY()));
                return diff == 0;
            }
        }
        return false;
    }

    /**
     * 清除动画
     *
     * @param recyclerView
     */
    public static void clearItemAnimator(@Nullable RecyclerView recyclerView) {
        if (recyclerView == null) {
            return;
        }
        if (recyclerView.getItemAnimator() != null) {
            recyclerView.getItemAnimator().setAddDuration(0);
            recyclerView.getItemAnimator().setChangeDuration(0);
            recyclerView.getItemAnimator().setMoveDuration(0);
            recyclerView.getItemAnimator().setRemoveDuration(0);
        }
        //关闭RV动画
        changeItemAnimation(recyclerView, false);
    }

    /**
     * 改变动画
     *
     * @param recyclerView
     */
    public static void changeItemAnimation(@Nullable RecyclerView recyclerView, boolean isAnimatorOpen) {
        if (recyclerView == null) {
            return;
        }
        // 关于RV动画的处理
        if (recyclerView.getItemAnimator() instanceof SimpleItemAnimator) {
            ((SimpleItemAnimator) recyclerView.getItemAnimator()).setSupportsChangeAnimations(isAnimatorOpen);
        }
    }

    public static RecyclerView.LayoutManager linear(Context context) {
        return new LinearLayoutManager(context);
    }

    public static RecyclerView.LayoutManager linearScroll(Context context, final boolean canScroll) {
        LinearLayoutManager manager = new LinearLayoutManager(context) {
            @Override
            public boolean canScrollHorizontally() {
                return canScroll && super.canScrollHorizontally();
            }

            @Override
            public boolean canScrollVertically() {
                return canScroll && super.canScrollVertically();
            }
        };
        return manager;
    }

    public static RecyclerView.LayoutManager linear(Context context, int orientation, boolean reverseLayout) {
        return new LinearLayoutManager(context, orientation, reverseLayout);
    }

    public static RecyclerView.LayoutManager grid(Context context, int spanCount) {
        return new GridLayoutManager(context, spanCount);
    }

    public static RecyclerView.LayoutManager grid(Context context, int spanCount, int orientation, boolean reverseLayout) {
        return new GridLayoutManager(context, spanCount, orientation, reverseLayout);
    }

    public static RecyclerView.LayoutManager staggeredGrid(int spanCount, int orientation) {
        return new StaggeredGridLayoutManager(spanCount, orientation);
    }

    /**
     * Recycler 滑动到指定的位置
     *
     * @param recyclerView view
     * @param snapMode     模式 start/end
     * @param position     指定位置
     */
    public static void smoothScrollToPosition(RecyclerView recyclerView, int snapMode, int position) {
        RecyclerView.LayoutManager layoutManager = recyclerView.getLayoutManager();
        if (layoutManager instanceof LinearLayoutManager) {
            LinearLayoutManager manager = (LinearLayoutManager) layoutManager;
            LinearSmoothScroller mScroller = null;
            if (snapMode == LinearSmoothScroller.SNAP_TO_START) {
                //开始位置
                mScroller = new TopSmoothScroller(recyclerView.getContext());
            } else if (snapMode == LinearSmoothScroller.SNAP_TO_END) {
                //结束位置
                mScroller = new BottomSmoothScroller(recyclerView.getContext());
            } else {
                //任意位置
                mScroller = new LinearSmoothScroller(recyclerView.getContext());
            }
            mScroller.setTargetPosition(position);
            manager.startSmoothScroll(mScroller);
        }
    }

    private static class TopSmoothScroller extends LinearSmoothScroller {
        TopSmoothScroller(Context context) {
            super(context);
        }

        @Override
        protected int getHorizontalSnapPreference() {
            return SNAP_TO_START;
        }

        @Override
        protected int getVerticalSnapPreference() {
            return SNAP_TO_START;
        }
    }

    private static class BottomSmoothScroller extends LinearSmoothScroller {
        BottomSmoothScroller(Context context) {
            super(context);
        }

        @Override
        protected int getHorizontalSnapPreference() {
            return SNAP_TO_END;
        }

        @Override
        protected int getVerticalSnapPreference() {
            return SNAP_TO_END;
        }
    }
}
