package com.winhc.business;

import android.text.TextUtils;

import com.google.gson.reflect.TypeToken;
import com.winhc.business.listeners.AccountInitialListener;
import com.winhc.business.listeners.OnAccountListener;
import com.winhc.business.model.LocalUserInfo;
import com.winhc.widget.util.CollectionUtils;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Copyright: 2016 Joindata. All rights reserved.
 * author：===>>     panic     <===
 * Create Time:   ===>>  2017/10/31 15:55  <===
 * email：panic4java@gmail.com
 * describe:
 * ********************************************************
 */

public class AccountManager {
    private static AccountManager sInstance;

    private LocalUserInfo mAccount = new LocalUserInfo();

    private boolean mIsInitial = false;

    private Object mLock = new byte[0];

    private final List<WeakReference<AccountInitialListener>> mInitialListeners = new ArrayList<>();

    private List<WeakReference<OnAccountListener>> mAccountListeners = new ArrayList<>();


    private AccountManager() {
        initAccount();
    }

    public static AccountManager getInstance() {
        if (sInstance == null) {
            synchronized (AccountManager.class) {
                if (sInstance == null) {
                    sInstance = new AccountManager();
                }
            }
        }
        return sInstance;
    }

    private void initAccount() {
        new Thread(new Runnable() {
            @Override
            public void run() {
                synchronized (mLock) {
                    mIsInitial = false;
                    mAccount = WinhcSharedPreferences.getBean(WinhcSharedPreferences.KEY_LOCAL_USER_INFO, new TypeToken<LocalUserInfo>() {
                    }.getType());
                    mIsInitial = true;
                    notifyInitialFinished();
                    mLock.notifyAll();
                }
            }
        }).start();
    }

    public void registerInitialListener(AccountInitialListener l) {
        if (l == null) {
            return;
        }
        synchronized (mInitialListeners) {
            mInitialListeners.add(new WeakReference<AccountInitialListener>(l));
        }
    }

    public void notifyInitialFinished() {
        synchronized (mInitialListeners) {
            if (!CollectionUtils.isEmpty(mInitialListeners)) {
                final Iterator iterator = mInitialListeners.iterator();
                while (iterator.hasNext()) {
                    final WeakReference<AccountInitialListener> initialReference =
                            (WeakReference<AccountInitialListener>) iterator.next();
                    if (initialReference != null) {
                        final AccountInitialListener listener = initialReference.get();
                        if (listener != null) {
                            listener.onInitialFinished(mAccount);
                        } else {
                            iterator.remove();
                        }
                    } else {
                        iterator.remove();
                    }
                }
            }
        }
    }

    public void registerAccountListener(OnAccountListener l) {
        if (l == null) {
            return;
        }
        synchronized (mAccountListeners) {
            mAccountListeners.add(new WeakReference<OnAccountListener>(l));
        }
    }

    public void notifyLoginSuccess(LocalUserInfo model) {
        synchronized (mAccountListeners) {
            if (!CollectionUtils.isEmpty(mAccountListeners)) {
                final Iterator iterator = mAccountListeners.iterator();
                while (iterator.hasNext()) {
                    final WeakReference<OnAccountListener> accountReference =
                            (WeakReference<OnAccountListener>) iterator.next();
                    if (accountReference != null) {
                        final OnAccountListener listener = accountReference.get();
                        if (listener != null) {
                            listener.onLoginSuccess(model);
                        }
                    } else {
                        iterator.remove();
                    }
                }
            }
        }
    }

    public void notifyLoginFailed(String msg) {
        synchronized (mAccountListeners) {
            if (!CollectionUtils.isEmpty(mAccountListeners)) {
                final Iterator iterator = mAccountListeners.iterator();
                while (iterator.hasNext()) {
                    final WeakReference<OnAccountListener> accountReference =
                            (WeakReference<OnAccountListener>) iterator.next();
                    if (accountReference != null) {
                        final OnAccountListener listener = accountReference.get();
                        if (listener != null) {
                            listener.onLoginFailed(msg);
                        }
                    } else {
                        iterator.remove();
                    }
                }
            }
        }
    }

    public void notifyLogout() {
        synchronized (mAccountListeners) {
            if (!CollectionUtils.isEmpty(mAccountListeners)) {
                final Iterator iterator = mAccountListeners.iterator();
                while (iterator.hasNext()) {
                    final WeakReference<OnAccountListener> accountReference =
                            (WeakReference<OnAccountListener>) iterator.next();
                    if (accountReference != null) {
                        final OnAccountListener listener = accountReference.get();
                        if (listener != null) {
                            listener.onAccountLogout();
                        }
                    } else {
                        iterator.remove();
                    }
                }
            }
        }
    }

    public boolean isLoginSync() {
        if (mIsInitial) {
            return isLogin();
        } else {
            synchronized (mLock) {
                while (!mIsInitial) {
                    try {
                        mLock.wait();
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
            }
            return isLogin();
        }
    }

    private boolean isLogin() {
        return mAccount != null && !TextUtils.isEmpty(mAccount.sessionId);
    }

    public LocalUserInfo getAccount() {
        if (mAccount == null) {
            mAccount = WinhcSharedPreferences.getBean(WinhcSharedPreferences.KEY_LOCAL_USER_INFO, new TypeToken<LocalUserInfo>() {
            }.getType());
        }
        return mAccount == null ? new LocalUserInfo() : mAccount;
    }

    public void clearCacheParams() {
        mAccount = new LocalUserInfo();
        WinhcSharedPreferences.resetByKey(WinhcSharedPreferences.KEY_LOCAL_USER_INFO);

    }

    public String getSessionId() {
        if (mAccount == null) {
            mAccount = WinhcSharedPreferences.getBean(WinhcSharedPreferences.KEY_LOCAL_USER_INFO, new TypeToken<LocalUserInfo>() {
            }.getType());
        }
        return mAccount == null ? "" : mAccount.sessionId;
    }

    public void saveAccount(String loginId, String sessionId, String userId, String hasLoginPwd, String hasPayPwd, String email, String avatar,
                            String mobileNo, String nickName, String userName, ArrayList<String> activityCodes, LocalUserInfo.UserIm userIm) {
        if (mAccount == null) {
            mAccount = new LocalUserInfo();
        }
        mAccount.setAllInfo(loginId, sessionId, userId, hasLoginPwd, hasPayPwd, email, avatar, mobileNo, nickName, userName, activityCodes, userIm);

        WinhcSharedPreferences.saveBean(WinhcSharedPreferences.KEY_LOCAL_USER_INFO, mAccount);//缓存本地数据
    }

    public void saveAccount(LocalUserInfo localUserInfo) {
        mAccount = localUserInfo;
        WinhcSharedPreferences.saveBean(WinhcSharedPreferences.KEY_LOCAL_USER_INFO, mAccount);//缓存本地修改过的数据
    }

    public boolean isLawAuthUser() {
        return mAccount != null && "2".equals(mAccount.userExt.lawyerStatus) && "2".equals(mAccount.userExt.getIdentity());
    }

    public String currentIdentity() {
        String result = "其他";
        if (null != mAccount && null != mAccount.userExt) {
            String identity = mAccount.userExt.getIdentity();
            if (TextUtils.isEmpty(identity)) result = "其他";
            else {
                switch (identity) {
                    case "1":
                        result = "普通用户";
                        break;
                    case "2":
                        result = "律师";
                        break;
                    case "3":
                        result = "企业法务";
                        break;
                    case "4":
                        result = "其他法律人";
                        break;
                }
            }
        }
        return result;
    }
}
