package com.winhc.business.net;

import android.os.Handler;
import android.os.Looper;

import com.winhc.business.BuildConfig;
import com.winhc.business.utils.GsonUtils;
import com.winhc.http.AppLogInterceptor;
import com.winhc.http.ResultCallback;
import com.winhc.http.ServiceFactory;

import java.io.Closeable;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.concurrent.TimeUnit;

import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.Dispatcher;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import retrofit2.Retrofit;
import retrofit2.adapter.rxjava2.RxJava2CallAdapterFactory;
import retrofit2.converter.gson.GsonConverterFactory;

/**
 * Create by LZ at 2021/9/7
 */
public class WinNet extends ServiceFactory {
    private static final int DEFAULT_TIMEOUT = 15; //默认超时时间
    private Handler handler = new Handler(Looper.getMainLooper());
    private OkHttpClient okHttpClient;

    private static class Holder {
        private static final WinNet winNet = new WinNet();
    }

    public static WinNet getInstance() {
        return Holder.winNet;
    }

    WinNet() {
        OkHttpClient.Builder builder = new OkHttpClient.Builder();
        builder.connectTimeout(DEFAULT_TIMEOUT, TimeUnit.SECONDS)
                .readTimeout(20, TimeUnit.SECONDS)
                .writeTimeout(DEFAULT_TIMEOUT, TimeUnit.SECONDS);
        // 添加一个请求拦截器
        builder.addInterceptor(new AppHeaderInterceptor());
        if (printLog()) {//日志输出
            AppLogInterceptor interceptor = new AppLogInterceptor();
            interceptor.setLevel(AppLogInterceptor.Level.BODY);
            builder.addInterceptor(interceptor);
        }
        okHttpClient = builder.build();
    }

    @Override
    public Retrofit.Builder buildRetrofit(String url, OkHttpClient httpClient) {
        return new Retrofit.Builder()
                .baseUrl(url)
                .client(httpClient)
                .addConverterFactory(GsonConverterFactory.create(GsonUtils.getGson()))
                .addCallAdapterFactory(RxJava2CallAdapterFactory.create());
    }

    @Override
    public OkHttpClient buildClient() {
        return okHttpClient;
    }

    @Override
    public String serviceUrl() {
        return BuildConfig.SERVER_URL;
    }

    /**
     * 下载文件并提供出进度监听     *
     *
     * @param url            地址
     * @param destDir        文件存放路径
     * @param resultCallback 回调
     */
    public void downAsync(String url, String tag, final String destDir, final ResultCallback<String> resultCallback) {
        if (okHttpClient == null) {
            return;
        }
        sendStartCallback(resultCallback);
        Request request = new Request.Builder()
                .tag(tag)
                .url(url)
                .build();
        okHttpClient.newCall(request).enqueue(new Callback() {
            @Override
            public void onFailure(Call call, IOException e) {
                sendFailureCallback(resultCallback, call, e);
            }

            @Override
            public void onResponse(Call call, Response response) throws IOException {
                InputStream is = null;
                byte[] buf = new byte[2048];
                int len;
                int sum = 0;
                FileOutputStream fos = null;
                try {
                    File file = new File(destDir);
                    if (!file.exists()) {
                        File parentFile = file.getParentFile();
                        if (parentFile == null) {
                            sendFailureCallback(resultCallback, call, new FileNotFoundException("文件路径找不到"));
                            return;
                        }
                        parentFile.mkdirs();
                        file.createNewFile();
                    }
                    long contentLength = response.body().contentLength();
                    is = response.body().byteStream();
                    fos = new FileOutputStream(file);
                    while ((len = is.read(buf)) != -1) {
                        sum += len;
                        fos.write(buf, 0, len);
                        sendProgressCallback(resultCallback, sum, contentLength);
                    }
                    sendStringCallback(resultCallback, destDir);
                    fos.flush();
                } catch (IOException e) {
                    sendFailureCallback(resultCallback, call, e);
                } finally {
                    close(is);
                    close(fos);
                }
            }
        });
    }

    private void close(Closeable io) {
        if (io != null) {
            try {
                io.close();
            } catch (IOException e) {
            }
        }
    }

    protected void sendStartCallback(final ResultCallback callback) {
        handler.post(new Runnable() {
            @Override
            public void run() {
                if (callback != null) {
                    callback.onStart();
                }
            }
        });
    }

    protected void sendFailureCallback(final ResultCallback callback, final Call call, final Exception e) {
        handler.post(new Runnable() {
            @Override
            public void run() {
                if (callback != null) {
                    callback.onFailure(call, e);
                    callback.onFinish();
                }
            }
        });

    }

    protected void sendStringCallback(final ResultCallback callback, final String json) {
        handler.post(new Runnable() {
            @Override
            public void run() {
                if (callback != null) {
                    callback.onSuccess(json);
                    callback.onFinish();
                }
            }
        });
    }

    protected void sendProgressCallback(final ResultCallback callback, final long byteRead, final long total) {
        handler.post(new Runnable() {
            @Override
            public void run() {
                if (callback != null) {
                    callback.onProgress(byteRead, total);
                }
            }
        });
    }

    /**
     * 全部队列中请求全部取消
     */
    public void cancelAll() {
        if (null != okHttpClient) {
            okHttpClient.dispatcher().cancelAll();
        }
    }

    /**
     * 根据标记取消请求的队列和排队中的队列
     *
     * @param tag
     */
    public void cancel(String tag) {
        if (null != okHttpClient) {
            Dispatcher dispatcher = okHttpClient.dispatcher();
            cancelCall(dispatcher.runningCalls(), tag);
            cancelCall(dispatcher.queuedCalls(), tag);
        }
    }

    private void cancelCall(List<Call> callList, String tag) {
        if (callList != null && callList.size() > 0) {
            for (Call call : callList) {
                if (call != null && tag.equals(call.request().tag())) {
                    call.cancel();
                }
            }
        }
    }


    @Override
    public boolean printLog() {
        return BuildConfig.DEBUG;
    }
}
